/****************************************************************************
File    : Helpers.cpp
/*
@(#) #SY# Atari800Win
@(#) #IS# Global helpers implementation
@(#) #BY# Richard Lawrence, Tomasz Szymankowski
@(#) #LM# 01.10.2000
*/

/*
Copyright (c) 2000 Tomasz Szymankowski

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; either 
version 2 of the License, or (at your option) any later version. This program is 
distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. You should have received a copy of the GNU
General Public License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "StdAfx.h"
#include "Atari800Win.h"
#include "Helpers.h"
#include "FileSmallDlg.h"


/////////////////////////////////////////////////////////////////////////////
// Windows key codes

int g_iNormKeys[] = {
/*  0*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_BREAK,		
/*  4*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*  8*/		AKEY_BACKSPACE,			AKEY_TAB,				AKEY_NONE,				AKEY_NONE,
/* 12*/		AKEY_CLEAR,				AKEY_RETURN,			AKEY_NONE,				AKEY_NONE,
/* 16*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_BREAK,
/* 20*/		AKEY_CAPSTOGGLE,		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 24*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_ESCAPE,
/* 28*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 32*/		AKEY_SPACE,				AKEY_CAPSLOCK,			AKEY_HELP|SPECIAL,		AKEY_ATARI,
/* 36*/		AKEY_CLEAR,				AKEY_LEFT,				AKEY_UP,				AKEY_RIGHT,
/* 40*/		AKEY_DOWN,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 44*/		AKEY_NONE,				AKEY_INSERT_CHAR,		AKEY_DELETE_CHAR,		AKEY_HELP|SPECIAL,
/* 48*/		AKEY_0,					AKEY_1,					AKEY_2,					AKEY_3,
/* 52*/		AKEY_4,					AKEY_5,					AKEY_6,					AKEY_7,
/* 56*/		AKEY_8,					AKEY_9,					AKEY_NONE,				AKEY_NONE,
/* 60*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 64*/		AKEY_NONE,				AKEY_a,					AKEY_b,					AKEY_c,
/* 68*/		AKEY_d,					AKEY_e,					AKEY_f,					AKEY_g,
/* 72*/		AKEY_h,					AKEY_i,					AKEY_j,					AKEY_k,
/* 76*/		AKEY_l,					AKEY_m,					AKEY_n,					AKEY_o,
/* 80*/		AKEY_p,					AKEY_q,					AKEY_r,					AKEY_s,
/* 84*/		AKEY_t,					AKEY_u,					AKEY_v,					AKEY_w,
/* 88*/		AKEY_x,					AKEY_y,					AKEY_z,					AKEY_ATARI,
/* 92*/		AKEY_ATARI,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 96*/		AKEY_0,					AKEY_1,					AKEY_2,					AKEY_3,
/*100*/		AKEY_4,					AKEY_5,					AKEY_6,					AKEY_7,
/*104*/		AKEY_8,					AKEY_9,					AKEY_ASTERISK,			AKEY_PLUS,
/*108*/		AKEY_NONE,				AKEY_MINUS,				AKEY_FULLSTOP,			AKEY_SLASH,
/*112*/		AKEY_NONE,				AKEY_OPTIONDWN,			AKEY_SELECTDWN,			AKEY_STARTDWN,
/*116*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*120*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*124*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*128*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*132*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*136*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*140*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*144*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*148*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*152*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*156*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*160*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*164*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*168*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*172*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*176*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*180*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*184*/		AKEY_NONE,				AKEY_NONE,				AKEY_SEMICOLON,			AKEY_EQUAL,
/*188*/		AKEY_COMMA,				AKEY_MINUS,				AKEY_FULLSTOP,			AKEY_SLASH,
/*192*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*196*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*200*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*204*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*208*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*212*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*216*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_BRACKETLEFT,
/*220*/		AKEY_BACKSLASH,			AKEY_BRACKETRIGHT,		AKEY_QUOTE,				AKEY_NONE,
/*224*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*228*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*232*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*236*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*240*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*244*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*248*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*252*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE
};

int g_iShftKeys[] = {
/*  0*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_BREAK|SHFT,		
/*  4*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*  8*/		AKEY_DELETE_CHAR,		AKEY_SETTAB,			AKEY_NONE,				AKEY_NONE,
/* 12*/		AKEY_CLEAR,				AKEY_RETURN|SHFT,		AKEY_NONE,				AKEY_NONE,
/* 16*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_BREAK,
/* 20*/		AKEY_CAPSTOGGLE|SHFT,	AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 24*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_ESCAPE|SHFT,
/* 28*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 32*/		AKEY_SPACE|SHFT,		AKEY_CAPSLOCK,			AKEY_HELP|SHFT|SPECIAL,	AKEY_ATARI|SHFT,
/* 36*/		AKEY_CLEAR,				AKEY_LEFT,				AKEY_UP,				AKEY_RIGHT,
/* 40*/		AKEY_DOWN,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 44*/		AKEY_NONE,				AKEY_INSERT_LINE,		AKEY_DELETE_LINE,		AKEY_HELP|SHFT|SPECIAL,
/* 48*/		AKEY_0|SHFT,			AKEY_1|SHFT,			AKEY_AT,				AKEY_3|SHFT,
/* 52*/		AKEY_4|SHFT,			AKEY_5|SHFT,			AKEY_CARET,				AKEY_AMPERSAND,
/* 56*/		AKEY_ASTERISK,			AKEY_9|SHFT,			AKEY_NONE,				AKEY_NONE,
/* 60*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 64*/		AKEY_NONE,				AKEY_A,					AKEY_B,					AKEY_C,
/* 68*/		AKEY_D,					AKEY_E,					AKEY_F,					AKEY_G,
/* 72*/		AKEY_H,					AKEY_I,					AKEY_J,					AKEY_K,
/* 76*/		AKEY_L,					AKEY_M,					AKEY_N,					AKEY_O,
/* 80*/		AKEY_P,					AKEY_Q,					AKEY_R,					AKEY_S,
/* 84*/		AKEY_T,					AKEY_U,					AKEY_V,					AKEY_W,
/* 88*/		AKEY_X,					AKEY_Y,					AKEY_Z,					AKEY_ATARI|SHFT,
/* 92*/		AKEY_ATARI|SHFT,		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 96*/		AKEY_0,					AKEY_1,					AKEY_2,					AKEY_3,
/*100*/		AKEY_4,					AKEY_5,					AKEY_6,					AKEY_7,
/*104*/		AKEY_8,					AKEY_9,					AKEY_ASTERISK,			AKEY_PLUS,
/*108*/		AKEY_NONE,				AKEY_MINUS,				AKEY_GREATER,			AKEY_SLASH,
/*112*/		AKEY_NONE,				AKEY_OPTIONDWN|SHFT,	AKEY_SELECTDWN|SHFT,	AKEY_STARTDWN|SHFT,
/*116*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*120*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*124*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*128*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*132*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*136*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*140*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*144*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*148*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*152*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*156*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*160*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*164*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*168*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*172*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*176*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*180*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*184*/		AKEY_NONE,				AKEY_NONE,				AKEY_COLON,				AKEY_PLUS,
/*188*/		AKEY_LESS,				AKEY_UNDERSCORE,		AKEY_GREATER,			AKEY_QUESTION,
/*192*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*196*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*200*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*204*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*208*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*212*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*216*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_BRACKETLEFT,
/*220*/		AKEY_BAR,				AKEY_BRACKETRIGHT,		AKEY_DBLQUOTE,			AKEY_NONE,
/*224*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*228*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*232*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*236*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*240*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*244*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*248*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*252*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE
};

int g_iCtrlKeys[] = {
/*  0*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_BREAK|CTRL,		
/*  4*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*  8*/		AKEY_BACKSPACE|CTRL,	AKEY_CLRTAB,			AKEY_NONE,				AKEY_NONE,
/* 12*/		AKEY_CLEAR,				AKEY_RETURN|CTRL,		AKEY_NONE,				AKEY_NONE,
/* 16*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_CTRL_1,
/* 20*/		AKEY_CAPSTOGGLE|CTRL,	AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 24*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_ESCAPE|CTRL,
/* 28*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 32*/		AKEY_SPACE|CTRL,		AKEY_CAPSLOCK|CTRL,		AKEY_HELP|CTRL|SPECIAL,	AKEY_ATARI|CTRL,
/* 36*/		AKEY_CLEAR,				AKEY_LEFT,				AKEY_UP,				AKEY_RIGHT,
/* 40*/		AKEY_DOWN,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 44*/		AKEY_NONE,				AKEY_INSERT_CHAR|CTRL,	AKEY_DELETE_CHAR|CTRL,	AKEY_HELP|CTRL|SPECIAL,
/* 48*/		AKEY_0|CTRL,			AKEY_1|CTRL,			AKEY_2|CTRL,			AKEY_3|CTRL,
/* 52*/		AKEY_4|CTRL,			AKEY_5|CTRL,			AKEY_6|CTRL,			AKEY_7|CTRL,
/* 56*/		AKEY_8|CTRL,			AKEY_9|CTRL,			AKEY_NONE,				AKEY_NONE,
/* 60*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 64*/		AKEY_NONE,				AKEY_CTRL_a,			AKEY_CTRL_b,			AKEY_CTRL_c,
/* 68*/		AKEY_CTRL_d,			AKEY_CTRL_e,			AKEY_CTRL_f,			AKEY_CTRL_g,
/* 72*/		AKEY_CTRL_h,			AKEY_CTRL_i,			AKEY_CTRL_j,			AKEY_CTRL_k,
/* 76*/		AKEY_CTRL_l,			AKEY_CTRL_m,			AKEY_CTRL_n,			AKEY_CTRL_o,
/* 80*/		AKEY_CTRL_p,			AKEY_CTRL_q,			AKEY_CTRL_r,			AKEY_CTRL_s,
/* 84*/		AKEY_CTRL_t,			AKEY_CTRL_u,			AKEY_CTRL_v,			AKEY_CTRL_w,
/* 88*/		AKEY_CTRL_x,			AKEY_CTRL_y,			AKEY_CTRL_z,			AKEY_ATARI|CTRL,
/* 92*/		AKEY_ATARI|CTRL,		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 96*/		AKEY_0,					AKEY_1,					AKEY_2,					AKEY_3,
/*100*/		AKEY_4,					AKEY_5,					AKEY_6,					AKEY_7,
/*104*/		AKEY_8,					AKEY_9,					AKEY_ASTERISK,			AKEY_PLUS,
/*108*/		AKEY_NONE,				AKEY_UNDERSCORE,		AKEY_FULLSTOP,			AKEY_SLASH,
/*112*/		AKEY_NONE,				AKEY_OPTIONDWN|CTRL,	AKEY_SELECTDWN|CTRL,	AKEY_STARTDWN|CTRL,
/*116*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*120*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*124*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*128*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*132*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*136*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*140*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*144*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*148*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*152*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*156*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*160*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*164*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*168*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*172*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*176*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*180*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*184*/		AKEY_NONE,				AKEY_NONE,				AKEY_SEMICOLON|CTRL,	AKEY_EQUAL|CTRL,
/*188*/		AKEY_COMMA|CTRL,		AKEY_MINUS|CTRL,		AKEY_FULLSTOP|CTRL,		AKEY_SLASH|CTRL,
/*192*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*196*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*200*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*204*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*208*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*212*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*216*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_BRACKETLEFT,
/*220*/		AKEY_BACKSLASH,			AKEY_BRACKETRIGHT,		AKEY_QUOTE,				AKEY_NONE,
/*224*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*228*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*232*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*236*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*240*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*244*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*248*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*252*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE
};

/* Following keys cannot be read with both shift and control pressed:
   J K L ; + * Z X C V B F1 F2 F3 F4 HELP */
int g_iShftCtrlKeys[] = {
/*  0*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_BREAK|CSFT,		
/*  4*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*  8*/		AKEY_BACKSPACE|CSFT,	AKEY_TAB|CSFT,			AKEY_NONE,				AKEY_NONE,
/* 12*/		AKEY_CLEAR|CSFT,		AKEY_RETURN|CSFT,		AKEY_NONE,				AKEY_NONE,
/* 16*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_CTRL_1,
/* 20*/		AKEY_CAPSTOGGLE|CSFT,	AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 24*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_ESCAPE,
/* 28*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 32*/		AKEY_SPACE|CSFT,		AKEY_CAPSLOCK|CSFT,		AKEY_HELP|CSFT|SPECIAL,	AKEY_ATARI,
/* 36*/		AKEY_CLEAR|CSFT,		AKEY_LEFT,				AKEY_UP,				AKEY_RIGHT,
/* 40*/		AKEY_DOWN,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 44*/		AKEY_NONE,				AKEY_INSERT_CHAR|CSFT,	AKEY_DELETE_CHAR|CSFT,	AKEY_NONE,
/* 48*/		AKEY_0|CSFT,			AKEY_1|CSFT,			AKEY_2|CSFT,			AKEY_3|CSFT,
/* 52*/		AKEY_4|CSFT,			AKEY_5|CSFT,			AKEY_6|CSFT,			AKEY_7|CSFT,
/* 56*/		AKEY_8|CSFT,			AKEY_9|CSFT,			AKEY_NONE,				AKEY_NONE,
/* 60*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 64*/		AKEY_NONE,				AKEY_a|CSFT,			AKEY_NONE,				AKEY_NONE,
/* 68*/		AKEY_d|CSFT,			AKEY_e|CSFT,			AKEY_f|CSFT,			AKEY_g|CSFT,
/* 72*/		AKEY_h|CSFT,			AKEY_i|CSFT,			AKEY_NONE,				AKEY_NONE,
/* 76*/		AKEY_NONE,				AKEY_m|CSFT,			AKEY_n|CSFT,			AKEY_o|CSFT,
/* 80*/		AKEY_p|CSFT,			AKEY_q|CSFT,			AKEY_r|CSFT,			AKEY_s|CSFT,
/* 84*/		AKEY_t|CSFT,			AKEY_u|CSFT,			AKEY_NONE,				AKEY_w|CSFT,
/* 88*/		AKEY_NONE,				AKEY_y|CSFT,			AKEY_NONE,				AKEY_ATARI,
/* 92*/		AKEY_ATARI|CSFT,		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/* 96*/		AKEY_0,					AKEY_1,					AKEY_2,					AKEY_3,
/*100*/		AKEY_4,					AKEY_5,					AKEY_6,					AKEY_7,
/*104*/		AKEY_8,					AKEY_9,					AKEY_ASTERISK,			AKEY_PLUS,
/*108*/		AKEY_NONE,				AKEY_MINUS,				AKEY_FULLSTOP,			AKEY_SLASH,
/*112*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*116*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*120*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*124*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*128*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*132*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*136*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*140*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*144*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*148*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*152*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*156*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*160*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*164*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*168*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*172*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*176*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*180*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*184*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_EQUAL|CSFT,
/*188*/		AKEY_COMMA|CSFT,		AKEY_MINUS|CSFT,		AKEY_FULLSTOP|CSFT,		AKEY_SLASH|CSFT,
/*192*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*196*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*200*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*204*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*208*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*212*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*216*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_BRACKETLEFT,
/*220*/		AKEY_BAR,				AKEY_BRACKETRIGHT,		AKEY_DBLQUOTE,			AKEY_NONE,
/*224*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*228*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*232*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*236*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*240*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*244*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*248*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE,
/*252*/		AKEY_NONE,				AKEY_NONE,				AKEY_NONE,				AKEY_NONE
};

int g_iToggleKeys[] = {
/*  0*/		0,	0,	0,	0,
/*  4*/		0,	0,	0,	0,
/*  8*/		0,	0,	0,	0,
/* 12*/		0,	0,	0,	0,
/* 16*/		1,	1,	0,	0,	/* Shift and Control */
/* 20*/		0,	0,	0,	0,
/* 24*/		0,	0,	0,	0,
/* 28*/		0,	0,	0,	0,
/* 32*/		0,	0,	0,	0,
/* 36*/		0,	0,	0,	0,
/* 40*/		0,	0,	0,	0,
/* 44*/		0,	0,	0,	0,
/* 48*/		0,	0,	0,	0,
/* 52*/		0,	0,	0,	0,
/* 56*/		0,	0,	0,	0,
/* 60*/		0,	0,	0,	0,
/* 64*/		0,	0,	0,	0,
/* 68*/		0,	0,	0,	0,
/* 72*/		0,	0,	0,	0,
/* 76*/		0,	0,	0,	0,
/* 80*/		0,	0,	0,	0,
/* 84*/		0,	0,	0,	0,
/* 88*/		0,	0,	0,	0,
/* 92*/		0,	0,	0,	0,
/* 96*/		0,	0,	0,	0,
/*100*/		0,	0,	0,	0,
/*104*/		0,	0,	0,	0,
/*108*/		0,	0,	0,	0,
/*112*/		0,	1,	1,	1,	/* Option, Select, Start */
/*116*/		0,	0,	0,	0,
/*120*/		0,	0,	0,	0,
/*124*/		0,	0,	0,	0,
/*128*/		0,	0,	0,	0,
/*132*/		0,	0,	0,	0,
/*136*/		0,	0,	0,	0,
/*140*/		0,	0,	0,	0,
/*144*/		0,	0,	0,	0,
/*148*/		0,	0,	0,	0,
/*152*/		0,	0,	0,	0,
/*156*/		0,	0,	0,	0,
/*160*/		0,	0,	0,	0,
/*164*/		0,	0,	0,	0,
/*168*/		0,	0,	0,	0,
/*172*/		0,	0,	0,	0,
/*176*/		0,	0,	0,	0,
/*180*/		0,	0,	0,	0,
/*184*/		0,	0,	0,	0,
/*188*/		0,	0,	0,	0,
/*192*/		0,	0,	0,	0,
/*196*/		0,	0,	0,	0,
/*200*/		0,	0,	0,	0,
/*204*/		0,	0,	0,	0,
/*208*/		0,	0,	0,	0,
/*212*/		0,	0,	0,	0,
/*216*/		0,	0,	0,	0,
/*220*/		0,	0,	0,	0,
/*224*/		0,	0,	0,	0,
/*228*/		0,	0,	0,	0,
/*232*/		0,	0,	0,	0,
/*236*/		0,	0,	0,	0,
/*240*/		0,	0,	0,	0,
/*244*/		0,	0,	0,	0,
/*248*/		0,	0,	0,	0,
/*252*/		0,	0,	0,	0
};


/////////////////////////////////////////////////////////////////////////////
// File management

LPCSTR  PF_DSK_FILTER =	"Atari disk images (*.atr,*.xfd, etc)|*.atr;*.xfd;*.atr.gz;*.atz;*.xfd.gz;*.xfz;*.dcm|ATR Images only (*.atr,*.atr.gz,*.atz)|*.atr;*.atr.gz;*.atz|XFD Images only (*.xfd,*.xfd.gz,*.xfz)|*.xfd;*.xfd.gz;*.xfz|DCM Images only (*.dcm)|*.dcm|All Files (*.*)|*.*||";
LPCSTR  PF_BIN_FILTER =	"Atari executable files (*.com,*.exe,*.xex)|*.com;*.exe;*.xex|All Files (*.*)|*.*||";
LPCSTR  PF_A8S_FILTER =	"Atari state images (*.a8s)|*.a8s|All Files (*.*)|*.*||";
LPCSTR  PF_A8K_FILTER =	"Atari keyboard templates (*.a8k)|*.a8k|All Files (*.*)|*.*||";
LPCSTR  PF_ACT_FILTER =	"Atari palette files (*.act)|*.act|All Files (*.*)|*.*||";
LPCSTR  PF_PCX_FILTER =	"Atari screenshot files (*.pcx)|*.pcx|All Files (*.*)|*.*||";
LPCSTR  PF_WAV_FILTER =	"Waveform audio data files (*.wav)|*.wav|All Files (*.*)|*.*||";
LPCSTR  PF_CRT_FILTER =	"Atari cartridge images (*.rom,*.bin,*.crt)|*.rom;*.bin;*.crt|ROM Images only(*.rom,*.bin)|*.rom;*.bin|CART Images only (*.crt)|*.crt|All Files (*.*)|*.*||";
LPCSTR  PF_ROM_FILTER =	"ROM Images (*.rom,*.bin)|*.rom;*.bin|All Files (*.*)|*.*||";
LPCSTR  PF_CAR_FILTER =	"CART Images (*.crt)|*.crt|All Files (*.*)|*.*||";
LPCSTR  PF_DCM_FILTER =	"DCM disk images (*.dcm)|*.dcm|All Files (*.*)|*.*||";

/*========================================================
Function : PickFileName
=========================================================*/
/* #FN#
   Displays file dialog and ask user for a file name */
BOOL
/* #AS#
   TRUE if the file was picked, otherwise FALSE */
PickFileName( BOOL   bOpenFileDialog,		 /* #IN# */
			  LPSTR  pszFileName,			 /* #IN/OUT# */
			  LPCSTR pszTitle,				 /* #IN# */
			  LPCSTR pszFilter,				 /* #IN# */
			  LPCSTR pszDefExt,				 /* #IN# */
			  DWORD  dwFlags,				 /* #IN# */
			  BOOL   bCheckPath  /*=TRUE*/,	 /* #IN#*/
			  LPCSTR pszNoneFile /*="None"*/ /* #IN#*/ )
{
	char szFilePath[ MAX_PATH ];
	int  nResult = IDOK;

	strncpy( szFilePath, pszFileName, MAX_PATH );
	szFilePath[ MAX_PATH - 1 ] = '\0';

	if( bCheckPath )
	{
		CFileStatus fsStatus;
		CFile       cfFile;
		int         i;

		/* Standard FileDialog wants that */
		do
		{
			/* Remove the ending backslash */
			for( i = strlen( szFilePath ) - 1; i >= 0 && szFilePath[ i ] == '\\'; i-- )
				szFilePath[ i ] = '\0';
			/* We have got a drive here, don't process */
			if( strlen( szFilePath ) == 2 && szFilePath[ 1 ] == ':' )
			{
				strcat( szFilePath, "\\" );
				strcat( szFilePath, pszNoneFile );
				break;
			}
			/* That's the static method, don't panic */
			if( cfFile.GetStatus( szFilePath, fsStatus ) )
			{
				if( fsStatus.m_attribute & CFile::directory )
				{
					strcat( szFilePath, "\\" );
					strcat( szFilePath, pszNoneFile );
				}
				break;
			}
			else
			{
				for( i = strlen( szFilePath );
					 i >= 0 && szFilePath[ i ] != '\\'; i-- )
					szFilePath[ i ] = '\0';
			}
		}
		while( i > 0 );
	}
	if( *szFilePath == '\0' )
		strcpy( szFilePath, pszNoneFile );

	/* Open the file dialog */
	if( g_ulScreenMode & SM_ATTR_SMALL_DLG )
	{
		CFileSmallDlg dlgFileSmall( pszTitle, szFilePath );

		if( IDOK == (nResult = dlgFileSmall.DoModal()) )
			strncpy( pszFileName, dlgFileSmall.GetPathName(), MAX_PATH );
	}
	else
	{
		CFileDialog	fdlgFile( bOpenFileDialog, pszDefExt, szFilePath,
			dwFlags | OFN_ENABLEHOOK, pszFilter, AfxGetMainWnd() );

		fdlgFile.m_ofn.lpstrTitle = pszTitle;
		fdlgFile.m_ofn.lpfnHook   = FileDialogHookProc;

		if( IDOK == (nResult = fdlgFile.DoModal()) )
			strncpy( pszFileName, fdlgFile.GetPathName(), MAX_PATH );
	}
	return (nResult == IDOK ? TRUE : FALSE);
} /* #OF# PickFileName */

/*========================================================
Function : PickFileName
=========================================================*/
/* #FN#
   Displays file dialog and ask user for a file name */
BOOL
/* #AS#
   TRUE if the file was picked, otherwise FALSE */
PickFileName( BOOL     bOpenFileDialog,		   /* #IN# */
			  CString &strFileName,			   /* #IN/OUT# */
			  LPCSTR   pszTitle,			   /* #IN# */
			  LPCSTR   pszFilter,			   /* #IN# */
			  LPCSTR   pszDefExt,			   /* #IN# */
			  DWORD    dwFlags,				   /* #IN# */
			  BOOL     bCheckPath  /*=TRUE*/,  /* #IN#*/
			  LPCSTR   pszNoneFile /*="None"*/ /* #IN#*/ )
{
	BOOL bResult =
		PickFileName( bOpenFileDialog, strFileName.GetBuffer( MAX_PATH ), pszTitle, pszFilter, pszDefExt, dwFlags, bCheckPath, pszNoneFile );
	strFileName.ReleaseBuffer();

	return bResult;
} /* #OF# PickFileName */


/////////////////////////////////////////////////////////////////////////////
// Cartridge management

/*========================================================
Function : PickCartridge
=========================================================*/
/* #FN#
   Displays file dialog and ask user for cartridge */
BOOL
/* #AS#
   TRUE if the cartridge was picked, otherwise FALSE */
PickCartridge( LPSTR pszCurrentCart )
{
	BOOL bResult = FALSE;

	if( PickFileName( TRUE, pszCurrentCart, "Select ROM to use as inserted cart",
					  PF_CRT_FILTER, "rom", PF_LOAD_FLAGS ) &&
		*pszCurrentCart != '\0' )
	{
		bResult = TRUE;
	}
	return bResult;
} /* #OF# PickCartridge */

/*========================================================
Function : AttachCartridge
=========================================================*/
/* #FN#
   Attaches cartridge */
BOOL
/* #AS#
   TRUE if the cartridge was attached, otherwise FALSE */
AttachCartridge( LPSTR pszCartridge, int nCartType )
{
	int nResult = 0;

	if( rom_inserted )
		Remove_ROM();

	if( NO_CART == nCartType )
		nCartType = FindCartType( pszCartridge, FALSE, TRUE );

	/* strcpy will be called within Insert_ROM functions */
	/* strcpy( g_szCurrentRom, pszCartridge ); */
	cart_type = NO_CART;

	if( NORMAL8_CART == nCartType &&
		(nResult = Insert_8K_ROM( pszCartridge )) )
	{
		cart_type = NORMAL8_CART;
	}
	else
	if( NORMAL16_CART == nCartType &&
		(nResult = Insert_16K_ROM( pszCartridge )) )
	{
		cart_type = NORMAL16_CART;
	}
	else
	if( OSS_SUPERCART == nCartType &&
		(nResult = Insert_OSS_ROM( pszCartridge )) )
	{
		cart_type = OSS_SUPERCART;
	}
	else
	if( AGS32_CART == nCartType &&
		(nResult = Insert_32K_5200ROM( pszCartridge )) )
	{
		cart_type = AGS32_CART;
	}
	else
	if( DB_SUPERCART == nCartType &&
		(nResult = Insert_DB_ROM( pszCartridge )) )
	{
		cart_type = DB_SUPERCART;
	}
	else
	if( CARTRIDGE == nCartType &&
		(nResult = Insert_Cartridge( pszCartridge )) )
	{
		cart_type = CARTRIDGE;
	}
	/* If there was an error... */
	if( !nResult )
	{
		DisplayMessage( IDS_ERROR_ROM_LOAD, 0, MB_ICONEXCLAMATION | MB_OK, pszCartridge );
		strcpy( g_szCurrentRom, "None" );
	}
	return (BOOL)nResult;
} /* #OF# AttachCartridge */


/////////////////////////////////////////////////////////////////////////////
// Miscellanous

/*========================================================
Function : RestartEmulation
=========================================================*/
/* #FN#
   Restarts the emulator (kernel and Windows stuff) */
void
/* #AS#
   Nothing */
RestartEmulation( BOOL bForceWinInit /*=TRUE*/,
				  BOOL bIgnoreParams /*=TRUE*/ )
{
	ULONG ulAtariState = g_ulAtariState;

	Clear_Temp_Files();

	/* Reinitialise the kernel */
	main( bIgnoreParams ? 1    : g_argc,
		  bIgnoreParams ? NULL : g_argv );

	RestartSound(); /* Restart sound if necessary */

	if( bForceWinInit ||
		/* Drag&Drop: don't worry about loosing the focus,
		   the emulated Atari won't start in this case */
		g_ulAtariState & ATARI_UNINITIALIZED )
	{
		RECT rc;
		GetClientRect( g_hMainWnd, &rc );
		rc.left = rc.right / 2; rc.right = rc.left + 1;
		rc.top = rc.bottom / 2; rc.bottom = rc.top + 1;

		g_hViewWnd = NULL;
		/* Reinitialise Windows stuff */
		RedrawWindow( g_hMainWnd, &rc, NULL, RDW_INVALIDATE /*RDW_NOFRAME | RDW_NOERASE | RDW_UPDATENOW*/ );
	}
	else
		g_ulAtariState = ulAtariState;

	ClearScreen( FALSE );
} /* #OF# RestartEmulation */

/*========================================================
Function : RefreshViewBounds
=========================================================*/
/* #FN#
   Cashes view area bondaries */
void
/* #AS#
   Nothing */
RefreshViewBounds( void )
{
	GetWindowRect( g_hViewWnd, &g_rcViewBounds );
	/* Deflate the rect boundaries */
	g_rcViewBounds.left += 2; g_rcViewBounds.right -= 2;
	g_rcViewBounds.top += 2; g_rcViewBounds.bottom -= 2;

	if( g_ulMiscStates & MS_MOUSE_CAPTURED )
	{
		/* Re-confine the cursor to a rectangular area on the view */
		ClipCursor( NULL );
		ClipCursor( &g_rcViewBounds );
		/* Initialise the mouse emulation settings */
		ResetMouse( MOUSE_JOY == g_nMouseMode &&
					Atari5200 == machine );
	}
} /* #OF# RefreshViewBounds */

/*========================================================
Function : HelpInfo
=========================================================*/
/* #FN#
   Context help handler of dialog windows */
void
/* #AS#
   Nothing */
HelpInfo( HELPINFO *pHelpInfo,
		  HWND      hDlgWnd,
		  UINT      nDlgWnd,
		  UINT      nFirstCtrl,
		  UINT      nLastCtrl )
{
	/* Check to see if the F1+SHIFT keystroke is used */
	if( !(GetAsyncKeyState( VK_F1 ) >> ((sizeof(SHORT) * 8) - 1)) || GetAsyncKeyState( VK_SHIFT ) >> ((sizeof(SHORT) * 8) - 1) )
	{
		HWND hCtrl   = WindowFromPoint( pHelpInfo->MousePos );
		UINT nCtrlId = (hCtrl ? GetDlgCtrlID( hCtrl ) : pHelpInfo->iCtrlId);

		/* Static and disabled windows are not indicated by
		   the WindowFromPoint method */
		if( hDlgWnd == hCtrl && pHelpInfo->hItemHandle )
		{
			CRect rc;
			GetWindowRect( (HWND)pHelpInfo->hItemHandle, rc );

			if( rc.PtInRect( pHelpInfo->MousePos ) )
				nCtrlId = pHelpInfo->iCtrlId;
		}
		if( nCtrlId &&
			nCtrlId >= nFirstCtrl &&
			nCtrlId <= nLastCtrl )
		{
			AfxGetApp()->WinHelp( HID_BASE_CONTROL + nCtrlId, HELP_CONTEXTPOPUP );
			return;
		}
	}
	AfxGetApp()->WinHelp( HID_BASE_RESOURCE + nDlgWnd );
} /* #OF# HelpInfo */

/*========================================================
Function : ShowEmulatorSpeed
=========================================================*/
/* #FN#
   Prints info about emulated system on the status bar */
void
/* #AS#
   Nothing */
ShowEmulatorSpeed( BOOL bForceShow,
				   int  nSpeed /*=-1*/ )
{
	CAtari800WinApp *pMainApp = (CAtari800WinApp *)AfxGetApp();
	if( pMainApp )
		pMainApp->ShowEmulatorSpeed( bForceShow, nSpeed );
} /* #OF# ShowEmulatorSpeed */
  
/*========================================================
Function : DescribeAtariSystem
=========================================================*/
/* #FN#
   Prints info about emulated system in main window */
void
/* #AS#
   Nothing */
DescribeAtariSystem( BOOL bForceShow /*=FALSE*/,
					 int  nSpeed /*=-1*/ )
{
	strcpy( g_szSystemDesc, g_ulScreenMode & SM_MODE_FULL ? "  " : "Atari800Win: " );

	if( g_ulAtariState & ATARI_UNINITIALIZED )
		strcat( g_szSystemDesc, TITLE_FAILURE );
	else
	if( g_ulAtariState & ATARI_RUNNING )
	{
		if( g_ulAtariState & ATARI_MONITOR )
			strcat( g_szSystemDesc, TITLE_MONITOR );
		else
		if( g_ulAtariState & ATARI_PAUSED )
			strcat( g_szSystemDesc, TITLE_PAUSED );
		else
		if( g_ulAtariState & ATARI_NOFOCUS )
			strcat( g_szSystemDesc, TITLE_STOPPED );
		else
		{
			/* What about the emulated system type? */
			switch( default_system )
			{
				case 1:
					strcat( g_szSystemDesc, "800 OS-A " );
					break;
				case 2:
					strcat( g_szSystemDesc, "800 OS-B " );
					break;
				case 3:
					strcat( g_szSystemDesc, "800XL " );
					break;
				case 4:
					strcat( g_szSystemDesc, "130XE " );
					break;
				case 5:
					strcat( g_szSystemDesc, "320XE " );
					if( 1 == Ram256 )
						strcat( g_szSystemDesc, "(Rambo) " );
					else
						strcat( g_szSystemDesc, "(Compy) " );
					break;
				case 6:
					strcat( g_szSystemDesc, "5200 " );
					break;
				default:
					strcat( g_szSystemDesc, "Unknown system! " );
					break;
			}
			/* What about the emulated TV mode? */
			switch( default_tv_mode )
			{
				case 1: /* PAL */
					if( g_ulMiscStates & MS_FULL_SPEED )
						strcat( g_szSystemDesc, "PAL Full Speed" );
					else
						strcat( g_szSystemDesc, "PAL 1.77MHz" );
					break;
					
				case 2: /* NTSC */
					if( g_ulMiscStates & MS_FULL_SPEED )
						strcat( g_szSystemDesc, "NTSC Full Speed" );
					else
						strcat( g_szSystemDesc, "NTSC 1.79MHz" );
					break;
					
				default:
					strcat( g_szSystemDesc, "Unknown TV type!" );
					break;
			}
			/* What about the SIO patch? */
			if( enable_sio_patch )
				strcat( g_szSystemDesc, " SIO" );
		}
	}
	else
		strcat( g_szSystemDesc, TITLE_NOTRUN );

	if( g_hMainWnd && g_ulScreenMode & SM_MODE_WIND )
	{
		SetWindowText( g_hMainWnd, g_szSystemDesc );
	}
	ShowEmulatorSpeed( bForceShow, nSpeed );
} /* #OF# DescribeAtariSystem */

/*========================================================
Function : FileDialogHookProc
=========================================================*/
/* #FN#
   Callback function used with the Explorer-style Open and Save As dialog boxes */
UINT CALLBACK
/* #AS#
   If zero, the default dialog box procedure processes the message */
FileDialogHookProc( HWND   hDlg,   /* #IN# Handle to child dialog window    */
					UINT   uMsg,   /* #IN# WM_NOTIFY                        */
					WPARAM wParam, /* #IN# Not used                         */
					LPARAM lParam  /* #IN# Message information (LPOFNOTIFY) */
)
{
	LPNMHDR lpNMHDR = (LPNMHDR)lParam;

	if( WM_NOTIFY == uMsg && lpNMHDR && CDN_INITDONE == lpNMHDR->code )
	{
		CWnd *pWnd = NULL;
		/* Unfortunately, the context help is available only in windowed modes */
		if( g_ulScreenMode & SM_MODE_FULL )
			SetWindowLong( lpNMHDR->hwndFrom, GWL_EXSTYLE, ~WS_EX_CONTEXTHELP & GetWindowLong( lpNMHDR->hwndFrom, GWL_EXSTYLE ) );

		pWnd = pWnd->FromHandle( lpNMHDR->hwndFrom );
		ASSERT(pWnd);
		pWnd->CenterWindow();
		return 1;
	}
	return 0;
} /* #OF# FileDialogHookProc */


BOOL g_bLargeFonts = LOWORD(GetDialogBaseUnits()) >= 10;
